import numpy as np
import pandas as pd
from plotly.offline import plot, iplot
import plotly.graph_objs as go
from scipy import stats

from phi_relative_lcmsms.helper import create_pivot

def calc_cohortwise_mean_std_sem(nacorr_merged_output_df, calc_identifier_df):
    """
    This function will calculate the mean, std and sem for the calculated identifiers
    :param nacorr_merged_output_df: The na correction output
    :param calc_identifier_df: The dataframe of calculated identifiers
    :return: A dataframes with calculated mean, std and sem for identifiers
    """

    metadata_df = nacorr_merged_output_df[['Sample', 'Replicate', 'Cohort Name']].drop_duplicates()
    cohort_sample_dict = dict(zip(metadata_df['Sample'], metadata_df['Cohort Name']))
    replicate_sample_dict = dict(zip(metadata_df['Sample'], metadata_df['Replicate']))
    melted_identifier_df = calc_identifier_df.melt('Identifier').rename(columns = {'variable': 'Sample'})
    melted_identifier_df['Replicate'] = melted_identifier_df['Sample'].map(replicate_sample_dict)
    melted_identifier_df['Cohort'] = melted_identifier_df['Sample'].map(cohort_sample_dict)
    pivoted_phi_mean = create_pivot(melted_identifier_df, "value", "Identifier", "Cohort", np.mean)
    pivoted_phi_mean = pivoted_phi_mean.melt("Identifier").rename(columns = {'value': 'mean'})
    pivoted_phi_std = create_pivot(melted_identifier_df, "value", "Identifier", "Cohort", np.std)
    pivoted_phi_std = pivoted_phi_std.melt("Identifier").rename(columns = {'value': 'std'})
    pivoted_phi_sem = create_pivot(melted_identifier_df, "value", "Identifier", "Cohort", lambda x: np.std(x.dropna())/np.sqrt(len(x.dropna())))
    pivoted_phi_sem = pivoted_phi_sem.melt("Identifier").rename(columns = {'value': 'sem'})
    identifier_mean_std_df = pd.merge(pivoted_phi_mean, pivoted_phi_std, on=['Identifier', 'Cohort'])
    identifier_mean_std_sem_df = pd.merge(identifier_mean_std_df, pivoted_phi_sem, on=['Identifier', 'Cohort'])
    
    return (identifier_mean_std_sem_df)

def identifier_bar_plot(identifier_cohortwise_df, identifier, cohorts_order = None):
    """
    This function will create a barplot for selected calculated identifier 
    :param identifier: The calculated identifier
    :param cohorts_order: The cohorts order
    :return: A dataframes with calculated mean, std and sem for identifiers
    """
    identifier_filtered_df = identifier_cohortwise_df[identifier_cohortwise_df['Identifier']==identifier]
    identifier_filtered_df = identifier_filtered_df.apply(lambda x: x.str.strip() if x.dtype == "object" else x)
    cohorts_list = identifier_filtered_df['Cohort'].unique().tolist()
    filtered_cohorts_list = cohorts_list
    common_cohorts = []
    if cohorts_order:
        cohorts_order = [cohort.strip() for cohort in cohorts_order]
        common_cohorts = [cohort for cohort in cohorts_order if cohort in cohorts_list]
    if common_cohorts:
        filtered_cohorts_list = common_cohorts
    
    data = []
    for cohort in filtered_cohorts_list:
        ident_filtered_cohortwise_df = identifier_filtered_df[identifier_filtered_df['Cohort']==cohort]
        x1 = ident_filtered_cohortwise_df['Identifier']
        y1 = ident_filtered_cohortwise_df['mean']
        y1_err = ident_filtered_cohortwise_df['sem']
        trace1 = go.Bar(
        x=x1,
        y=y1, 
        error_y=dict(type='data', array=y1_err, visible=True),
        name= cohort,
        )
        data.append(trace1)
    layout = go.Layout(title = identifier,
                           xaxis=dict(title=identifier, titlefont=dict(size=14)),
                           yaxis=dict(title='Fractional Enrichments',titlefont=dict(size=14)),
                           barmode='group')
    fig = go.Figure(data=data, layout=layout)
    iplot(fig, filename="identifier_bar_plot")

def make_identifier_sample_to_cohort(nacorr_merged_output_df, calc_identifier_df, cohorts_order = None):
    """
    This function will make the sample to cohort matrix format for all identifiers
    :param nacorr_merged_output_df: The na correction output
    :param calc_identifier_df: The dataframe of calculated identifiers
    :param cohorts_order: The cohorts order
    :return: A dataframes with sample to cohort matrix format for all identifiers
    """

    metadata_df = nacorr_merged_output_df[['Sample', 'Replicate', 'Cohort Name']].drop_duplicates()
    metadata_df = metadata_df.apply(lambda x: x.str.strip() if x.dtype == "object" else x)
    cohort_sample_dict = dict(zip(metadata_df['Sample'], metadata_df['Cohort Name']))
    replicate_sample_dict = dict(zip(metadata_df['Sample'], metadata_df['Replicate']))
    melted_identifier_df = calc_identifier_df.melt('Identifier').rename(columns = {'variable': 'Sample'})
    melted_identifier_df['Replicate'] = melted_identifier_df['Sample'].map(replicate_sample_dict)
    melted_identifier_df['Cohort'] = melted_identifier_df['Sample'].map(cohort_sample_dict)
    ident_sample_to_cohort_df = create_pivot(melted_identifier_df, 'value', ['Identifier','Replicate'], 'Cohort', func="first")
    
    cohorts_list = melted_identifier_df['Cohort'].unique().tolist()
    common_cohorts = []
    if cohorts_order:
        cohorts_order = [cohort.strip() for cohort in cohorts_order]
        common_cohorts = [cohort for cohort in cohorts_order if cohort in cohorts_list]
    if common_cohorts:
        cols_order = ['Identifier','Replicate'] + common_cohorts
        ident_sample_to_cohort_df = ident_sample_to_cohort_df[cols_order]
        
    return (ident_sample_to_cohort_df)